@@ -2,14 +2,14 @@
 // SPDX-FileCopyrightText: 2021 Carl Schwan <carl@carlschwan.eu>
 // SPDX-License-Identifier: GPL-3.0-only
 
-#include "abstractaccount.h"
+#include "account/abstractaccount.h"
 
-#include "accountmanager.h"
-#include "messagefiltercontainer.h"
-#include "navigation.h"
-#include "networkcontroller.h"
-#include "relationship.h"
+#include "account/accountmanager.h"
+#include "account/relationship.h"
+#include "network/networkcontroller.h"
 #include "tokodon_debug.h"
+#include "utils/messagefiltercontainer.h"
+#include "utils/navigation.h"
 
 #include <KLocalizedString>
 
@@ -117,7 +117,7 @@ QUrl AbstractAccount::apiUrl(const QString &path) const
     return url;
 }
 
-void AbstractAccount::registerApplication(const QString &appName, const QString &website, const QString &additionalScopes)
+void AbstractAccount::registerApplication(const QString &appName, const QString &website, const QString &additionalScopes, const bool useAuthCode)
 {
     // clear any previous bearer token credentials
     m_token = QString();
@@ -130,11 +130,13 @@ void AbstractAccount::registerApplication(const QString &appName, const QString
     // Store for future usage (e.g. authorizeUrl)
     m_additionalScopes = ourAdditionalScopes + additionalScopes;
 
+    m_redirectUri = useAuthCode ? QStringLiteral("urn:ietf:wg:oauth:2.0:oob") : QStringLiteral("tokodon://oauth");
+
     // register
     const QUrl regUrl = apiUrl(QStringLiteral("/api/v1/apps"));
     const QJsonObject obj{
         {QStringLiteral("client_name"), appName},
-        {QStringLiteral("redirect_uris"), QStringLiteral("tokodon://oauth")},
+        {QStringLiteral("redirect_uris"), m_redirectUri},
         {QStringLiteral("scopes"), QStringLiteral("read write follow %1").arg(m_additionalScopes)},
         {QStringLiteral("website"), website},
     };
@@ -302,7 +304,7 @@ QUrl AbstractAccount::getAuthorizeUrl() const
     QUrl url = apiUrl(QStringLiteral("/oauth/authorize"));
     QUrlQuery q = buildOAuthQuery();
 
-    q.addQueryItem(QStringLiteral("redirect_uri"), QStringLiteral("tokodon://oauth"));
+    q.addQueryItem(QStringLiteral("redirect_uri"), m_redirectUri);
     q.addQueryItem(QStringLiteral("response_type"), QStringLiteral("code"));
     q.addQueryItem(QStringLiteral("scope"), QStringLiteral("read write follow ") + m_additionalScopes);
 
@@ -345,7 +347,7 @@ void AbstractAccount::setToken(const QString &authcode)
     QUrlQuery q = buildOAuthQuery();
 
     q.addQueryItem(QStringLiteral("client_secret"), m_client_secret);
-    q.addQueryItem(QStringLiteral("redirect_uri"), QStringLiteral("tokodon://oauth"));
+    q.addQueryItem(QStringLiteral("redirect_uri"), m_redirectUri);
     q.addQueryItem(QStringLiteral("grant_type"), QStringLiteral("authorization_code"));
     q.addQueryItem(QStringLiteral("code"), authcode);
 
@@ -618,8 +620,8 @@ void AbstractAccount::handleNotification(const QJsonDocument &doc)
     std::shared_ptr<Notification> n = std::make_shared<Notification>(this, obj);
 
     if (n->type() == Notification::FollowRequest) {
-        m_hasFollowRequests = true;
-        Q_EMIT hasFollowRequestsChanged();
+        m_followRequestCount++;
+        Q_EMIT followRequestCountChanged();
     }
 
     Q_EMIT notification(n);
@@ -676,6 +678,11 @@ void AbstractAccount::executeAction(Identity *identity, AccountAction accountAct
     });
 }
 
+int AbstractAccount::followRequestCount() const
+{
+    return m_followRequestCount;
+}
+
 void AbstractAccount::followAccount(Identity *identity, bool reblogs, bool notify)
 {
     executeAction(identity,
@@ -745,7 +752,7 @@ void AbstractAccount::mutateRemotePost(const QString &url, const QString &verb)
                 // TODO: we can't delete this immediately, will need some smarter cleanup in the PostEditorBackend
                 Post *post = new Post(this, this);
                 post->fromJson(status);
-                Q_EMIT Navigation::instance().replyTo(post->postId(), post->mentions(), post->visibility(), post->getAuthorIdentity(), post);
+                Q_EMIT Navigation::instance().replyTo(post->postId(), post->mentions(), post->visibility(), post->authorIdentity().get(), post);
             } else {
                 const QString localID = status["id"_L1].toString();
                 mutatePost(localID, verb);
