@@ -5,8 +5,8 @@
 
 #include "account/accountmanager.h"
 #include "account/profileeditor.h"
-#include "helperreply.h"
-#include "mockaccount.h"
+#include "autotests/helperreply.h"
+#include "autotests/mockaccount.h"
 
 class ProfileEditorTest : public QObject
 {
