@@ -4,7 +4,7 @@
 
 #pragma once
 
-#include "post.h"
+#include "timeline/post.h"
 
 class QTimer;
 class AbstractAccount;
@@ -26,6 +26,8 @@ public:
     QHash<int, QByteArray> roleNames() const override;
     const QList<Attachment *> &attachments() const;
 
+    void copyFromOther(AttachmentEditorModel *other);
+
 public Q_SLOTS:
     QNetworkReply *append(const QString &fileName);
     void appendExisting(Attachment *attachment);
