@@ -1,12 +1,11 @@
 // SPDX-FileCopyrightText: 2022 Carl Schwan <carl@carlschwan.eu>
 // SPDX-License-Identifier: LGPL-2.1-or-later
 
-#include "posteditorbackend.h"
+#include "editor/posteditorbackend.h"
 
-#include "abstractaccount.h"
-#include "accountmanager.h"
-#include "attachmenteditormodel.h"
-#include "utils.h"
+#include "account/abstractaccount.h"
+#include "account/accountmanager.h"
+#include "editor/attachmenteditormodel.h"
 
 #include <KLocalizedString>
 
@@ -185,6 +184,22 @@ int PostEditorBackend::charactersLeft() const
     return m_account->maxPostLength() - m_status.length() + sum;
 }
 
+void PostEditorBackend::copyFromOther(PostEditorBackend *other)
+{
+    setId(other->id());
+    setSpoilerText(other->spoilerText());
+    setVisibility(other->visibility());
+    setLanguage(other->language());
+    setScheduledAt(other->scheduledAt());
+    setMentions(other->mentions());
+    setSensitive(other->sensitive());
+    setAccount(other->account());
+    m_pollEnabled = other->m_pollEnabled;
+    Q_EMIT pollEnabledChanged();
+    m_poll->copyFromOther(other->m_poll);
+    m_attachmentEditorModel->copyFromOther(other->m_attachmentEditorModel);
+}
+
 QJsonDocument PostEditorBackend::toJsonDocument() const
 {
     QJsonObject obj;
@@ -192,7 +207,7 @@ QJsonDocument PostEditorBackend::toJsonDocument() const
     obj["spoiler_text"_L1] = m_spoilerText;
     obj["status"_L1] = m_status;
     obj["sensitive"_L1] = m_sensitive;
-    obj["visibility"_L1] = visibilityToString(m_visibility);
+    obj["visibility"_L1] = Post::visibilityToString(m_visibility);
 
     if (!m_inReplyTo.isEmpty()) {
         obj["in_reply_to_id"_L1] = m_inReplyTo;
