@@ -3,8 +3,8 @@
 
 #pragma once
 
-#include "polleditorbackend.h"
-#include "post.h"
+#include "editor/polleditorbackend.h"
+#include "timeline/post.h"
 
 class AttachmentEditorModel;
 
@@ -66,6 +66,8 @@ public:
 
     int charactersLeft() const;
 
+    Q_INVOKABLE void copyFromOther(PostEditorBackend *other);
+
 public Q_SLOTS:
     void save();
     void edit();
