@@ -1,128 +1,30 @@
 // SPDX-FileCopyrightText: 2021 kaniini <https://git.pleroma.social/kaniini>
+// SPDX-FileCopyrightText: 2024 Joshua Goins <josh@redstrate.com>
 // SPDX-License-Identifier: GPL-3.0-only
 
-#include "post.h"
+#include "timeline/post.h"
 
 #include "account/abstractaccount.h"
-#include "utils/utils.h"
+#include "utils/texthandler.h"
 
 #include <KLocalizedString>
 
 using namespace Qt::Literals::StringLiterals;
 
-static QMap<QString, Attachment::AttachmentType> stringToAttachmentType = {
-    {QStringLiteral("image"), Attachment::AttachmentType::Image},
-    {QStringLiteral("gifv"), Attachment::AttachmentType::GifV},
-    {QStringLiteral("video"), Attachment::AttachmentType::Video},
-    {QStringLiteral("unknown"), Attachment::AttachmentType::Unknown},
+static const QMap<Post::Visibility, QString> p_visibilityToString = {
+    {Post::Visibility::Public, QStringLiteral("public")},
+    {Post::Visibility::Unlisted, QStringLiteral("unlisted")},
+    {Post::Visibility::Private, QStringLiteral("private")},
+    {Post::Visibility::Direct, QStringLiteral("direct")},
 };
 
-Attachment::Attachment(QObject *parent)
-    : QObject(parent)
-{
-}
-
-Attachment::Attachment(const QJsonObject &obj, QObject *parent)
-    : QObject(parent)
-{
-    fromJson(obj);
-}
-
-void Attachment::fromJson(const QJsonObject &obj)
-{
-    if (!obj.contains("type"_L1)) {
-        m_type = Unknown;
-        return;
-    }
-
-    m_id = obj["id"_L1].toString();
-    m_url = obj["url"_L1].toString();
-    m_preview_url = obj["preview_url"_L1].toString();
-    m_remote_url = obj["remote_url"_L1].toString();
-
-    setDescription(obj["description"_L1].toString());
-    m_blurhash = obj["blurhash"_L1].toString();
-    m_sourceHeight = obj["meta"_L1].toObject()["original"_L1].toObject()["height"_L1].toInt();
-    m_sourceWidth = obj["meta"_L1].toObject()["original"_L1].toObject()["width"_L1].toInt();
-
-    // determine type if we can
-    const auto type = obj["type"_L1].toString();
-    if (stringToAttachmentType.contains(type)) {
-        m_type = stringToAttachmentType[type];
-    }
-
-    if (!m_remote_url.isEmpty()) {
-        const auto mimeType = QMimeDatabase().mimeTypeForFile(m_remote_url);
-        if (m_type == AttachmentType::Unknown && mimeType.name().contains("image"_L1)) {
-            m_type = AttachmentType::Image;
-        }
-    }
-
-    if (obj.contains("meta"_L1) && obj["meta"_L1].toObject().contains("focus"_L1)) {
-        m_focusX = obj["meta"_L1].toObject()["focus"_L1].toObject()["x"_L1].toDouble();
-        m_focusY = obj["meta"_L1].toObject()["focus"_L1].toObject()["y"_L1].toDouble();
-    }
-}
-
-QString Post::type() const
-{
-    return QStringLiteral("post");
-}
-
-QString Attachment::description() const
-{
-    return m_description;
-}
-
-void Attachment::setDescription(const QString &description)
-{
-    m_description = description;
-}
-
-QString Attachment::id() const
-{
-    return m_id;
-}
-
-int Attachment::isVideo() const
-{
-    if (m_type == AttachmentType::GifV || m_type == AttachmentType::Video) {
-        return 1;
-    }
-
-    return 0;
-}
-
-QString Attachment::tempSource() const
-{
-    return QStringLiteral("image://blurhash/%1").arg(m_blurhash);
-}
-
-double Attachment::focusX() const
-{
-    return m_focusX;
-}
-
-void Attachment::setFocusX(double value)
-{
-    if (value != m_focusX) {
-        m_focusX = value;
-        Q_EMIT focusXChanged();
-    }
-}
-
-double Attachment::focusY() const
-{
-    return m_focusY;
-}
-
-void Attachment::setFocusY(double value)
-{
-    if (value != m_focusY) {
-        m_focusY = value;
-        Q_EMIT focusYChanged();
-    }
-}
+static const QMap<QString, Post::Visibility> p_stringToVisibility = {
+    {QStringLiteral("public"), Post::Visibility::Public},
+    {QStringLiteral("unlisted"), Post::Visibility::Unlisted},
+    {QStringLiteral("private"), Post::Visibility::Private},
+    {QStringLiteral("direct"), Post::Visibility::Direct},
+    {QStringLiteral("local"), Post::Visibility::Local},
+};
 
 Post::Post(AbstractAccount *account, QObject *parent)
     : QObject(parent)
@@ -133,39 +35,6 @@ Post::Post(AbstractAccount *account, QObject *parent)
     m_visibility = stringToVisibility(visibilityString);
 }
 
-QString computeContent(const QJsonObject &obj, std::shared_ptr<Identity> authorIdentity)
-{
-    const auto emojis = CustomEmoji::parseCustomEmojis(obj["emojis"_L1].toArray());
-    QString content = CustomEmoji::replaceCustomEmojis(emojis, obj["content"_L1].toString());
-
-    const auto tags = obj["tags"_L1].toArray();
-    const QString baseUrl = authorIdentity->url().toDisplayString(QUrl::RemovePath);
-
-    for (const auto &tag : tags) {
-        const auto tagObj = tag.toObject();
-
-        const QList<QString> tagFormats = {
-            QStringLiteral("tags"), // Mastodon
-            QStringLiteral("tag") // Akkoma/Pleroma
-        };
-
-        for (const QString &tagFormat : tagFormats) {
-            content = content.replace(baseUrl + QStringLiteral("/%1/").arg(tagFormat) + tagObj["name"_L1].toString(),
-                                      QStringLiteral("hashtag:/") + tagObj["name"_L1].toString(),
-                                      Qt::CaseInsensitive);
-        }
-    }
-
-    const auto mentions = obj["mentions"_L1].toArray();
-
-    for (const auto &mention : mentions) {
-        const auto mentionObj = mention.toObject();
-        content = content.replace(mentionObj["url"_L1].toString(), QStringLiteral("account:/") + mentionObj["id"_L1].toString(), Qt::CaseInsensitive);
-    }
-
-    return content;
-}
-
 Post::Post(AbstractAccount *account, QJsonObject obj, QObject *parent)
     : QObject(parent)
     , m_parent(account)
@@ -175,15 +44,6 @@ Post::Post(AbstractAccount *account, QJsonObject obj, QObject *parent)
     fromJson(obj);
 }
 
-Post *Notification::createPost(AbstractAccount *account, const QJsonObject &obj, QObject *parent)
-{
-    if (!obj.empty()) {
-        return new Post(account, obj, parent);
-    }
-
-    return nullptr;
-}
-
 void Post::fromJson(QJsonObject obj)
 {
     const auto accountDoc = obj["account"_L1].toObject();
@@ -211,13 +71,7 @@ void Post::fromJson(QJsonObject obj)
 
     m_spoilerText = obj["spoiler_text"_L1].toString();
 
-    // First process HTML for links, and custom emojis
-    const QString computedContent = computeContent(obj, m_authorIdentity);
-
-    // And then parse the content for standalone tags
-    auto [processedContent, standaloneTags] = parseContent(computedContent);
-    m_content = processedContent;
-    m_standaloneTags = standaloneTags;
+    processContent(obj);
 
     m_replyTargetId = obj["in_reply_to_id"_L1].toString();
 
@@ -306,74 +160,16 @@ void Post::fromJson(QJsonObject obj)
     if (obj.contains(QStringLiteral("poll")) && !obj[QStringLiteral("poll")].isNull()) {
         m_poll = std::make_unique<Poll>(obj[QStringLiteral("poll")].toObject());
     }
-
-    m_attachments_visible = !m_sensitive;
 }
 
-void Post::addAttachments(const QJsonArray &attachments)
+QString Post::postId() const
 {
-    for (const auto &attachment : attachments) {
-        m_attachments.append(new Attachment{attachment.toObject(), this});
-    }
+    return m_postId;
 }
 
-void Post::addAttachment(const QJsonObject &attachment)
+QString Post::originalPostId() const
 {
-    auto att = new Attachment{attachment, this};
-    if (att->m_url.isEmpty()) {
-        return;
-    }
-    m_attachments.append(att);
-
-    Q_EMIT attachmentUploaded();
-}
-
-void Post::setInReplyTo(const QString &inReplyTo)
-{
-    if (inReplyTo == m_replyTargetId) {
-        return;
-    }
-    m_replyTargetId = inReplyTo;
-    Q_EMIT inReplyToChanged();
-}
-
-int Post::repliesCount() const
-{
-    return m_repliesCount;
-}
-
-QString Post::inReplyTo() const
-{
-    return m_replyTargetId;
-}
-
-void Post::setDirtyAttachment()
-{
-    m_parent->invalidatePost(this);
-}
-
-QStringList Post::mentions() const
-{
-    return m_mentions;
-}
-
-QStringList Post::filters() const
-{
-    return m_filters;
-}
-
-QUrl Post::url() const
-{
-    return m_url;
-}
-
-void Post::setMentions(const QStringList &mentions)
-{
-    if (mentions == m_mentions) {
-        return;
-    }
-    m_mentions = mentions;
-    Q_EMIT mentionsChanged();
+    return m_originalPostId;
 }
 
 QDateTime Post::publishedAt() const
@@ -416,14 +212,64 @@ QString Post::absoluteTime() const
     return QLocale::system().toString(publishedAt(), QLocale::LongFormat);
 }
 
-QString Post::editedAt() const
+std::shared_ptr<Identity> Post::authorIdentity() const
 {
-    return QLocale::system().toString(m_editedAt, QLocale::ShortFormat);
+    return m_authorIdentity;
 }
 
-bool Post::wasEdited() const
+QString Post::content() const
 {
-    return m_editedAt.isValid();
+    return m_content;
+}
+
+QString Post::plainContent() const
+{
+    return m_plainContent;
+}
+
+bool Post::hasContent() const
+{
+    return m_hasContent;
+}
+
+Post::Visibility Post::visibility() const
+{
+    return m_visibility;
+}
+
+bool Post::sensitive() const
+{
+    return m_sensitive;
+}
+
+QString Post::spoilerText() const
+{
+    return m_spoilerText;
+}
+
+QList<Attachment *> Post::attachments() const
+{
+    return m_attachments;
+}
+
+std::optional<Application> Post::application() const
+{
+    return m_application;
+}
+
+QStringList Post::mentions() const
+{
+    return m_mentions;
+}
+
+QVector<QString> Post::standaloneTags() const
+{
+    return m_standaloneTags;
+}
+
+int Post::repliesCount() const
+{
+    return m_repliesCount;
 }
 
 int Post::favouritesCount() const
@@ -436,143 +282,30 @@ int Post::reblogsCount() const
     return m_reblogsCount;
 }
 
-static QMap<QString, Notification::Type> str_to_not_type = {
-    {QStringLiteral("favourite"), Notification::Type::Favorite},
-    {QStringLiteral("follow"), Notification::Type::Follow},
-    {QStringLiteral("mention"), Notification::Type::Mention},
-    {QStringLiteral("reblog"), Notification::Type::Repeat},
-    {QStringLiteral("update"), Notification::Type::Update},
-    {QStringLiteral("poll"), Notification::Type::Poll},
-    {QStringLiteral("status"), Notification::Type::Status},
-    {QStringLiteral("follow_request"), Notification::Type::FollowRequest},
-};
-
-Notification::Notification(AbstractAccount *account, const QJsonObject &obj, QObject *parent)
-    : m_account(account)
+QUrl Post::url() const
 {
-    const auto accountObj = obj["account"_L1].toObject();
-    const auto status = obj["status"_L1].toObject();
-    const auto accountId = accountObj["id"_L1].toString();
-    const auto type = obj["type"_L1].toString();
-
-    m_post = createPost(m_account, status, parent);
-    m_identity = m_account->identityLookup(accountId, accountObj);
-    m_type = str_to_not_type[type];
-    m_id = obj["id"_L1].toString().toInt();
+    return m_url;
 }
 
-int Notification::id() const
+QString Post::inReplyTo() const
 {
-    return m_id;
+    return m_replyTargetId;
 }
 
-AbstractAccount *Notification::account() const
+std::shared_ptr<Identity> Post::replyIdentity() const
 {
-    return m_account;
+    return m_replyIdentity;
 }
 
-Notification::Type Notification::type() const
+Poll *Post::poll() const
 {
-    return m_type;
+    return m_poll.get();
 }
 
-Post *Notification::post() const
+void Post::setPollJson(const QJsonObject &object)
 {
-    return m_post;
-}
-
-std::shared_ptr<Identity> Notification::identity() const
-{
-    return m_identity;
-}
-
-QString Post::spoilerText() const
-{
-    return m_spoilerText;
-}
-
-void Post::setSpoilerText(const QString &spoilerText)
-{
-    if (spoilerText == m_spoilerText) {
-        return;
-    }
-    m_spoilerText = spoilerText;
-    Q_EMIT spoilerTextChanged();
-}
-
-QString Post::content() const
-{
-    return m_content;
-}
-
-void Post::setContent(const QString &content)
-{
-    if (content == m_content) {
-        return;
-    }
-    m_content = content;
-    Q_EMIT contentChanged();
-}
-
-QVector<QString> Post::standaloneTags() const
-{
-    return m_standaloneTags;
-}
-
-QString Post::contentType() const
-{
-    return m_content_type;
-}
-
-void Post::setContentType(const QString &contentType)
-{
-    if (m_content_type == contentType) {
-        return;
-    }
-    m_content_type = contentType;
-    Q_EMIT contentTypeChanged();
-}
-
-bool Post::sensitive() const
-{
-    return m_sensitive;
-}
-
-void Post::setSensitive(bool sensitive)
-{
-    if (m_sensitive == sensitive) {
-        return;
-    }
-    m_sensitive = sensitive;
-    Q_EMIT sensitiveChanged();
-}
-
-Post::Visibility Post::visibility() const
-{
-    return m_visibility;
-}
-
-void Post::setVisibility(Visibility visibility)
-{
-    if (visibility == m_visibility) {
-        return;
-    }
-    m_visibility = visibility;
-    Q_EMIT visibilityChanged();
-}
-
-QString Post::language() const
-{
-    return m_language;
-}
-
-void Post::setLanguage(const QString &language)
-{
-    if (language == m_language) {
-        return;
-    }
-    m_language = language;
-    Q_EMIT languageChanged();
+    m_poll = std::make_unique<Poll>(object);
+    Q_EMIT pollChanged();
 }
 
 std::optional<Card> Post::card() const
@@ -580,28 +313,29 @@ std::optional<Card> Post::card() const
     return m_card;
 }
 
-Card *Post::getCard() const
+QString Post::language() const
 {
-    if (m_card.has_value()) {
-        return const_cast<Card *>(&m_card.value());
-    } else {
-        return nullptr;
-    }
+    return m_language;
 }
 
-void Post::setCard(std::optional<Card> card)
+bool Post::wasEdited() const
 {
-    m_card = card;
+    return m_editedAt.isValid();
 }
 
-std::optional<Application> Post::application() const
+QString Post::editedAt() const
 {
-    return m_application;
+    return QLocale::system().toString(m_editedAt, QLocale::ShortFormat);
 }
 
-void Post::setApplication(std::optional<Application> application)
+bool Post::boosted() const
 {
-    m_application = application;
+    return m_boosted;
+}
+
+std::shared_ptr<Identity> Post::boostIdentity() const
+{
+    return m_boostIdentity;
 }
 
 bool Post::favourited() const
@@ -624,6 +358,16 @@ void Post::setReblogged(bool reblogged)
     m_reblogged = reblogged;
 }
 
+bool Post::bookmarked() const
+{
+    return m_bookmarked;
+}
+
+void Post::setBookmarked(bool bookmarked)
+{
+    m_bookmarked = bookmarked;
+}
+
 bool Post::muted() const
 {
     return m_muted;
@@ -634,14 +378,14 @@ void Post::setMuted(bool muted)
     m_muted = muted;
 }
 
-bool Post::bookmarked() const
+QStringList Post::filters() const
 {
-    return m_bookmarked;
+    return m_filters;
 }
 
-void Post::setBookmarked(bool bookmarked)
+bool Post::hidden() const
 {
-    m_bookmarked = bookmarked;
+    return m_hidden;
 }
 
 bool Post::pinned() const
@@ -659,9 +403,31 @@ bool Post::filtered() const
     return m_filtered;
 }
 
-QList<Attachment *> Post::attachments() const
+void Post::addAttachments(const QJsonArray &attachments)
 {
-    return m_attachments;
+    for (const auto &attachment : attachments) {
+        m_attachments.append(new Attachment{attachment.toObject(), this});
+    }
+}
+
+QString Post::visibilityToString(Post::Visibility visibility)
+{
+    return p_visibilityToString[visibility];
+}
+
+Post::Visibility Post::stringToVisibility(const QString &visibility)
+{
+    return p_stringToVisibility[visibility];
+}
+
+QString Post::type() const
+{
+    return QStringLiteral("post");
+}
+
+Identity *Post::getAuthorIdentity() const
+{
+    return authorIdentity().get();
 }
 
 QQmlListProperty<Attachment> Post::attachmentList() const
@@ -669,19 +435,72 @@ QQmlListProperty<Attachment> Post::attachmentList() const
     return m_attachmentList;
 }
 
-void Post::setAttachmentsVisible(bool attachmentsVisible)
+Card *Post::getCard() const
 {
-    m_attachments_visible = attachmentsVisible;
+    if (m_card.has_value()) {
+        return const_cast<Card *>(&m_card.value());
+    } else {
+        return nullptr;
+    }
 }
 
-bool Post::attachmentsVisible() const
+void Post::setCard(std::optional<Card> card)
 {
-    return m_attachments_visible;
+    m_card = card;
 }
 
-bool Post::boosted() const
+void Post::setApplication(std::optional<Application> application)
 {
-    return m_boosted;
+    m_application = application;
+}
+
+void Post::processContent(const QJsonObject &obj)
+{
+    const QString originalHtml = obj["content"_L1].toString();
+
+    // First, replace custom emojis with their HTML representations
+    const auto emojis = CustomEmoji::parseCustomEmojis(obj["emojis"_L1].toArray());
+    QString processedHtml = TextHandler::replaceCustomEmojis(emojis, originalHtml);
+
+    // Then turn hashtags into proper links, so they link inside Tokodon
+    const auto tags = obj["tags"_L1].toArray();
+    const QString baseUrl = m_authorIdentity->url().toDisplayString(QUrl::RemovePath);
+
+    for (const auto &tag : tags) {
+        const auto tagObj = tag.toObject();
+
+        // The "url" field in the tag object is for our own instance,
+        // but the url for the tag in the HTML we're given is for their instance. Hence, the odd search & replace done here.
+        const QList<QString> tagFormats = {
+            QStringLiteral("tags"), // Mastodon
+            QStringLiteral("tag") // Akkoma/Pleroma
+        };
+
+        for (const QString &tagFormat : tagFormats) {
+            processedHtml = processedHtml.replace(baseUrl + QStringLiteral("/%1/").arg(tagFormat) + tagObj["name"_L1].toString(),
+                                                  QStringLiteral("hashtag:/") + tagObj["name"_L1].toString(),
+                                                  Qt::CaseInsensitive);
+        }
+    }
+
+    // Do the same for mentions
+    const auto mentions = obj["mentions"_L1].toArray();
+
+    for (const auto &mention : mentions) {
+        const auto mentionObj = mention.toObject();
+        processedHtml =
+            processedHtml.replace(mentionObj["url"_L1].toString(), QStringLiteral("account:/") + mentionObj["id"_L1].toString(), Qt::CaseInsensitive);
+    }
+
+    // Remove the standalone tags from the main content
+    auto [standaloneContent, standaloneTags] = TextHandler::removeStandaloneTags(processedHtml);
+    m_standaloneTags = standaloneTags;
+
+    m_hasContent = !standaloneContent.isEmpty();
+
+    // Finally, fix the bidirectionality of text if needed. See BUG 475043 for more details.
+    m_content = TextHandler::fixBidirectionality(standaloneContent);
+    m_plainContent = standaloneContent;
 }
 
 Card::Card(QJsonObject card)
@@ -773,114 +592,4 @@ QUrl Application::website() const
     return QUrl::fromUserInput(m_application[QLatin1String("website")].toString());
 }
 
-Identity *Post::getAuthorIdentity() const
-{
-    return authorIdentity().get();
-}
-
-std::shared_ptr<Identity> Post::authorIdentity() const
-{
-    return m_authorIdentity;
-}
-
-std::shared_ptr<Identity> Post::boostIdentity() const
-{
-    return m_boostIdentity;
-}
-
-std::shared_ptr<Identity> Post::replyIdentity() const
-{
-    return m_replyIdentity;
-}
-
-Poll *Post::poll() const
-{
-    return m_poll.get();
-}
-
-void Post::setPollJson(const QJsonObject &object)
-{
-    m_poll = std::make_unique<Poll>(object);
-    Q_EMIT pollChanged();
-}
-
-QString Post::postId() const
-{
-    return m_postId;
-}
-
-QString Post::originalPostId() const
-{
-    return m_originalPostId;
-}
-
-bool Post::isEmpty() const
-{
-    return m_postId.isEmpty();
-}
-
-QPair<QString, QList<QString>> Post::parseContent(const QString &html)
-{
-    const QRegularExpression hashtagExp(QStringLiteral("(?:<a\\b[^>]*>#<span>(\\S*)<\\/span><\\/a>)"));
-    const QRegularExpression extraneousParagraph(QStringLiteral("(\\s*(?:<(?:p|br)\\s*\\/?>)+\\s*<\\/p>)"));
-
-    QList<QString> standaloneTags;
-
-    // Find the last <p> or <br>
-    const int lastBreak = html.lastIndexOf(QStringLiteral("<br>"));
-    int lastParagraphBegin = html.lastIndexOf(QStringLiteral("<p>"));
-    if (lastBreak > lastParagraphBegin) {
-        lastParagraphBegin = lastBreak;
-    }
-
-    const int lastParagraphEnd = html.lastIndexOf(QStringLiteral("</p>"));
-    QString lastParagraph = html.mid(lastParagraphBegin, lastParagraphEnd - html.length());
-
-    QString processedHtml = html;
-
-    // Catch all the tags in the last paragraph of the post, but only if they are not surrounded by text
-    {
-        QList<QString> possibleTags;
-        QString possibleLastParagraph = lastParagraph;
-
-        auto matchIterator = hashtagExp.globalMatch(possibleLastParagraph);
-        while (matchIterator.hasNext()) {
-            const QRegularExpressionMatch match = matchIterator.next();
-            possibleTags.push_back(match.captured(1));
-            possibleLastParagraph = possibleLastParagraph.replace(match.captured(0), QStringLiteral(""));
-        }
-
-        // If this paragraph is truly extraneous, then we can take its tags, otherwise skip.
-        auto extraneousIterator = extraneousParagraph.globalMatch(possibleLastParagraph);
-        if (extraneousIterator.hasNext()) {
-            processedHtml.replace(lastParagraph, possibleLastParagraph);
-            standaloneTags = possibleTags;
-        }
-    }
-
-    const QRegularExpression extraneousBreakExp(QStringLiteral("(\\s*(?:<br\\s*\\/?>)+\\s*)<\\/p>"));
-
-    // Ensure we remove any remaining <br>'s which will mess up the spacing in a post.
-    // Example: "<p>Yosemite Valley reflections with rock<br />    </p>"
-    {
-        auto matchIterator = extraneousBreakExp.globalMatch(processedHtml);
-        while (matchIterator.hasNext()) {
-            const QRegularExpressionMatch match = matchIterator.next();
-            processedHtml = processedHtml.replace(match.captured(1), QStringLiteral(""));
-        }
-    }
-
-    // Ensure we remove any empty <p>'s which will mess up the spacing in a post.
-    // Example: "<p>Boris Karloff (again) as Imhotep</p><p>  </p>"
-    {
-        auto matchIterator = extraneousParagraph.globalMatch(processedHtml);
-        while (matchIterator.hasNext()) {
-            const QRegularExpressionMatch match = matchIterator.next();
-            processedHtml = processedHtml.replace(match.captured(1), QStringLiteral(""));
-        }
-    }
-
-    return {processedHtml, standaloneTags};
-}
-
 #include "moc_post.cpp"
