@@ -1,9 +1,8 @@
 // SPDX-FileCopyrightText: 2023 Joshua Goins <josh@redstrate.com>
 // SPDX-License-Identifier: LGPL-2.0-or-later
 
-#include "windowcontroller.h"
+#include "utils/windowcontroller.h"
 
-#include <KConfig>
 #include <KSharedConfig>
 #include <KWindowConfig>
 
@@ -34,3 +33,5 @@ void WindowController::saveGeometry()
 
     stateConfig->sync();
 }
+
+#include "moc_windowcontroller.cpp"
\ No newline at end of file
